@@ -93,22 +93,13 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static void
 parentize_widget (BonoboUIToolbar *toolbar,
-		  GtkWidget *widget)
+		  GtkWidget       *widget)
 {
 	g_assert (widget->parent == NULL);
 
 	/* The following is done according to the Bible, widget_system.txt, IV, 1.  */
 
 	gtk_widget_set_parent (widget, GTK_WIDGET (toolbar));
-
-	if (GTK_WIDGET_REALIZED (toolbar) && ! GTK_WIDGET_REALIZED (widget))
-		gtk_widget_realize (widget);
-
-	if (GTK_WIDGET_MAPPED (toolbar) && ! GTK_WIDGET_MAPPED (widget) &&  GTK_WIDGET_VISIBLE (widget))
-		gtk_widget_map (widget);
-
-	if (GTK_WIDGET_MAPPED (widget))
-		gtk_widget_queue_resize (GTK_WIDGET (toolbar));
 }
 
 static void
@@ -160,6 +151,7 @@ item_destroy_cb (GtkObject *object,
 
 	widget = GTK_WIDGET (object);
 	priv->items = g_list_remove (priv->items, object);
+	g_object_unref (object);
 }
 
 static void
@@ -701,18 +693,21 @@ impl_dispose (GObject *object)
 {
 	BonoboUIToolbar *toolbar;
 	BonoboUIToolbarPrivate *priv;
-	GList *p, *next;
+	GList *items, *p, *next;
 
 	toolbar = BONOBO_UI_TOOLBAR (object);
 	priv = toolbar->priv;
 
-	for (p = priv->items; p != NULL; p = next) {
+	items = priv->items;
+	for (p = items; p; p = next) {
 		GtkWidget *item_widget;
 
 		next = p->next;
 		item_widget = GTK_WIDGET (p->data);
-		if (item_widget->parent == NULL)
+		if (item_widget->parent == NULL) {
+			items = g_list_remove (items, item_widget);
 			gtk_widget_destroy (item_widget);
+		}
 	}
 
 	if (priv->popup_item &&
@@ -734,12 +729,9 @@ impl_dispose (GObject *object)
 static void
 impl_finalize (GObject *object)
 {
-	BonoboUIToolbar *toolbar;
-
-	toolbar = BONOBO_UI_TOOLBAR (object);
-
-	g_list_free (toolbar->priv->items);
+	BonoboUIToolbar *toolbar = (BonoboUIToolbar *) object;
 	
+	g_list_free (toolbar->priv->items);
 	g_free (toolbar->priv);
 
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
@@ -1148,8 +1140,9 @@ bonobo_ui_toolbar_class_init (BonoboUIToolbarClass *toolbar_class)
 static void
 bonobo_ui_toolbar_instance_init (BonoboUIToolbar *toolbar)
 {
-	BonoboUIToolbarPrivate *priv;
+	AtkObject *ao;
 	BonoboUIToolbarStyle style;
+	BonoboUIToolbarPrivate *priv;
 
 	GTK_WIDGET_SET_FLAGS (toolbar, GTK_NO_WINDOW);
 
@@ -1175,6 +1168,10 @@ bonobo_ui_toolbar_instance_init (BonoboUIToolbar *toolbar)
 	priv->tooltips                    = gtk_tooltips_new ();
 
 	toolbar->priv = priv;
+
+	ao = gtk_widget_get_accessible (GTK_WIDGET (toolbar));
+	if (ao)
+		atk_object_set_role (ao, ATK_ROLE_TOOL_BAR);
 }
 
 void
@@ -1284,18 +1281,21 @@ bonobo_ui_toolbar_insert (BonoboUIToolbar *toolbar,
 	g_return_if_fail (BONOBO_IS_UI_TOOLBAR (toolbar));
 	g_return_if_fail (BONOBO_IS_UI_TOOLBAR_ITEM (item));
 
-	g_object_ref (item);
-	gtk_object_sink (GTK_OBJECT (item));
-
 	priv = toolbar->priv;
 
 	/*
 	 *  This ugly hack is here since we might have unparented
 	 * a widget and then re-added it to the toolbar at a later
 	 * date, and un-parenting doesn't work quite properly yet.
+	 *
+	 *  Un-parenting is down to the widget possibly being a
+	 * child of either this widget, or the popup window.
 	 */
-	if (!g_list_find (priv->items, item))
+	if (!g_list_find (priv->items, item)) {
+		g_object_ref (item);
+		gtk_object_sink (GTK_OBJECT (item));
 		priv->items = g_list_insert (priv->items, item, position);
+	}
 
 	g_signal_connect_object (
 		item, "destroy",
@@ -1310,12 +1310,16 @@ bonobo_ui_toolbar_insert (BonoboUIToolbar *toolbar,
 		G_CALLBACK (item_set_want_label_cb),
 		toolbar, 0);
 
+	g_object_ref (toolbar);
+	g_object_ref (item);
+
 	set_attributes_on_child (item, priv->orientation, priv->style);
 	parentize_widget (toolbar, GTK_WIDGET (item));
 
-	g_assert (GTK_WIDGET (item)->parent == GTK_WIDGET (toolbar));
-
 	gtk_widget_queue_resize (GTK_WIDGET (toolbar));
+
+	g_object_unref (item);
+	g_object_unref (toolbar);
 }
 
 GList *
